@@ -54,9 +54,16 @@ typedef enum readstat_alignment_e {
 
 typedef enum readstat_compress_e {
     READSTAT_COMPRESS_NONE,
-    READSTAT_COMPRESS_ROWS
+    READSTAT_COMPRESS_ROWS,
+    READSTAT_COMPRESS_BINARY
 } readstat_compress_t;
 
+typedef enum readstat_endian_e {
+    READSTAT_ENDIAN_NONE,
+    READSTAT_ENDIAN_LITTLE,
+    READSTAT_ENDIAN_BIG
+} readstat_endian_t;
+
 typedef enum readstat_error_e {
     READSTAT_OK,
     READSTAT_ERROR_OPEN = 1,
@@ -87,18 +94,47 @@ typedef enum readstat_error_e {
     READSTAT_ERROR_NAME_CONTAINS_ILLEGAL_CHARACTER,
     READSTAT_ERROR_NAME_IS_RESERVED_WORD,
     READSTAT_ERROR_NAME_IS_TOO_LONG,
-    READSTAT_ERROR_BAD_TIMESTAMP,
+    READSTAT_ERROR_BAD_TIMESTAMP_STRING,
     READSTAT_ERROR_BAD_FREQUENCY_WEIGHT,
     READSTAT_ERROR_TOO_MANY_MISSING_VALUE_DEFINITIONS,
     READSTAT_ERROR_NOTE_IS_TOO_LONG,
     READSTAT_ERROR_STRING_REFS_NOT_SUPPORTED,
     READSTAT_ERROR_STRING_REF_IS_REQUIRED,
     READSTAT_ERROR_ROW_IS_TOO_WIDE_FOR_PAGE,
-    READSTAT_ERROR_ROW_IS_EMPTY
+    READSTAT_ERROR_TOO_FEW_COLUMNS,
+    READSTAT_ERROR_TOO_MANY_COLUMNS,
+    READSTAT_ERROR_NAME_IS_ZERO_LENGTH,
+    READSTAT_ERROR_BAD_TIMESTAMP_VALUE
 } readstat_error_t;
 
 const char *readstat_error_message(readstat_error_t error_code);
 
+typedef struct readstat_metadata_s {
+    int64_t     row_count;
+    int64_t     var_count;
+    time_t      creation_time;
+    time_t      modified_time;
+    int64_t     file_format_version;
+    readstat_compress_t compression;
+    readstat_endian_t endianness;
+    const char *table_name;
+    const char *file_label;
+    const char *file_encoding;
+    unsigned int is64bit:1;
+} readstat_metadata_t;
+
+int readstat_get_row_count(readstat_metadata_t *metadata);
+int readstat_get_var_count(readstat_metadata_t *metadata);
+time_t readstat_get_creation_time(readstat_metadata_t *metadata);
+time_t readstat_get_modified_time(readstat_metadata_t *metadata);
+int readstat_get_file_format_version(readstat_metadata_t *metadata);
+int readstat_get_file_format_is_64bit(readstat_metadata_t *metadata);
+readstat_compress_t readstat_get_compression(readstat_metadata_t *metadata);
+readstat_endian_t readstat_get_endianness(readstat_metadata_t *metadata);
+const char *readstat_get_table_name(readstat_metadata_t *metadata);
+const char *readstat_get_file_label(readstat_metadata_t *metadata);
+const char *readstat_get_file_encoding(readstat_metadata_t *metadata);
+
 typedef struct readstat_value_s {
     union {
         float       float_value;
@@ -164,6 +200,26 @@ typedef struct readstat_variable_s {
     int                     index_after_skipping;
 } readstat_variable_t;
 
+typedef struct readstat_schema_entry_s {
+    int                 row;
+    int                 col;
+    int                 len;
+    int                 skip;
+    readstat_variable_t variable;
+    char                labelset[32];
+    char                decimal_separator;
+} readstat_schema_entry_t;
+
+typedef struct readstat_schema_s {
+    char                    filename[255];
+    int                     rows_per_observation;
+    int                     cols_per_observation;
+    int                     first_line;
+    int                     entry_count;
+    char                    field_delimiter;
+    readstat_schema_entry_t *entries;
+} readstat_schema_t;
+
 /* Value accessors */
 readstat_type_t readstat_value_type(readstat_value_t value);
 readstat_type_class_t readstat_value_type_class(readstat_value_t value);
@@ -227,9 +283,7 @@ readstat_value_t readstat_variable_get_missing_range_hi(const readstat_variable_
 /* If the variable handler returns READSTAT_HANDLER_SKIP_VARIABLE, the value handler will not be called on
  * the associated variable. (Note that subsequent variables will retain their original index values.)
  */
-typedef int (*readstat_info_handler)(int obs_count, int var_count, void *ctx);
-typedef int (*readstat_metadata_handler)(const char *file_label, const char *orig_encoding,
-        time_t timestamp, long format_version, void *ctx);
+typedef int (*readstat_metadata_handler)(readstat_metadata_t *metadata, void *ctx);
 typedef int (*readstat_note_handler)(int note_index, const char *note, void *ctx);
 typedef int (*readstat_variable_handler)(int index, readstat_variable_t *variable, 
         const char *val_labels, void *ctx);
@@ -271,27 +325,30 @@ typedef struct readstat_io_s {
     int                            io_ctx_needs_free;
 } readstat_io_t;
 
+typedef struct readstat_callbacks_s {
+    readstat_metadata_handler      metadata;
+    readstat_note_handler          note;
+    readstat_variable_handler      variable;
+    readstat_fweight_handler       fweight;
+    readstat_value_handler         value;
+    readstat_value_label_handler   value_label;
+    readstat_error_handler         error;
+    readstat_progress_handler      progress;
+} readstat_callbacks_t;
+
 typedef struct readstat_parser_s {
-    readstat_info_handler          info_handler;
-    readstat_metadata_handler      metadata_handler;
-    readstat_note_handler          note_handler;
-    readstat_variable_handler      variable_handler;
-    readstat_fweight_handler       fweight_handler;
-    readstat_value_handler         value_handler;
-    readstat_value_label_handler   value_label_handler;
-    readstat_error_handler         error_handler;
-    readstat_progress_handler      progress_handler;
-    readstat_io_t                 *io;
-    const char                    *input_encoding;
-    const char                    *output_encoding;
-    long                           row_limit;
+    readstat_callbacks_t    handlers;
+    readstat_io_t          *io;
+    const char             *input_encoding;
+    const char             *output_encoding;
+    long                    row_limit;
+    long                    row_offset;
 } readstat_parser_t;
 
 readstat_parser_t *readstat_parser_init(void);
 void readstat_parser_free(readstat_parser_t *parser);
 void readstat_io_free(readstat_io_t *io);
 
-readstat_error_t readstat_set_info_handler(readstat_parser_t *parser, readstat_info_handler info_handler);
 readstat_error_t readstat_set_metadata_handler(readstat_parser_t *parser, readstat_metadata_handler metadata_handler);
 readstat_error_t readstat_set_note_handler(readstat_parser_t *parser, readstat_note_handler note_handler);
 readstat_error_t readstat_set_variable_handler(readstat_parser_t *parser, readstat_variable_handler variable_handler);
@@ -317,7 +374,9 @@ readstat_error_t readstat_set_file_character_encoding(readstat_parser_t *parser,
 readstat_error_t readstat_set_handler_character_encoding(readstat_parser_t *parser, const char *encoding);
 
 readstat_error_t readstat_set_row_limit(readstat_parser_t *parser, long row_limit);
+readstat_error_t readstat_set_row_offset(readstat_parser_t *parser, long row_offset);
 
+/* Parse binary / portable files */
 readstat_error_t readstat_parse_dta(readstat_parser_t *parser, const char *path, void *user_ctx);
 readstat_error_t readstat_parse_sav(readstat_parser_t *parser, const char *path, void *user_ctx);
 readstat_error_t readstat_parse_por(readstat_parser_t *parser, const char *path, void *user_ctx);
@@ -325,6 +384,20 @@ readstat_error_t readstat_parse_sas7bdat(readstat_parser_t *parser, const char *
 readstat_error_t readstat_parse_sas7bcat(readstat_parser_t *parser, const char *path, void *user_ctx);
 readstat_error_t readstat_parse_xport(readstat_parser_t *parser, const char *path, void *user_ctx);
 
+/* Parse a schema file... */
+readstat_schema_t *readstat_parse_sas_commands(readstat_parser_t *parser,
+    const char *filepath, void *user_ctx, readstat_error_t *outError);
+readstat_schema_t *readstat_parse_spss_commands(readstat_parser_t *parser,
+    const char *filepath, void *user_ctx, readstat_error_t *outError);
+readstat_schema_t *readstat_parse_stata_dictionary(readstat_parser_t *parser,
+    const char *filepath, void *user_ctx, readstat_error_t *outError);
+
+/* ... then pass the schema to the plain-text parser ... */
+readstat_error_t readstat_parse_txt(readstat_parser_t *parser, const char *filename, 
+        readstat_schema_t *schema, void *user_ctx);
+
+/* ... and free the schema structure */
+void readstat_schema_free(readstat_schema_t *schema);
 
 /* Internal module callbacks */
 typedef struct readstat_string_ref_s {
@@ -335,7 +408,7 @@ typedef struct readstat_string_ref_s {
 } readstat_string_ref_t;
 
 typedef size_t (*readstat_variable_width_callback)(readstat_type_t type, size_t user_width);
-typedef readstat_error_t (*readstat_variable_ok_callback)(readstat_variable_t *variable);
+typedef readstat_error_t (*readstat_variable_ok_callback)(const readstat_variable_t *variable);
 
 typedef readstat_error_t (*readstat_write_int8_callback)(void *row_data, const readstat_variable_t *variable, int8_t value);
 typedef readstat_error_t (*readstat_write_int16_callback)(void *row_data, const readstat_variable_t *variable, int16_t value);
@@ -351,6 +424,7 @@ typedef readstat_error_t (*readstat_begin_data_callback)(void *writer);
 typedef readstat_error_t (*readstat_write_row_callback)(void *writer, void *row_data, size_t row_len);
 typedef readstat_error_t (*readstat_end_data_callback)(void *writer);
 typedef void (*readstat_module_ctx_free_callback)(void *module_ctx);
+typedef readstat_error_t (*readstat_metadata_ok_callback)(void *writer);
 
 typedef struct readstat_writer_callbacks_s {
     readstat_variable_width_callback    variable_width;
@@ -369,6 +443,7 @@ typedef struct readstat_writer_callbacks_s {
     readstat_write_row_callback         write_row;
     readstat_end_data_callback          end_data;
     readstat_module_ctx_free_callback   module_ctx_free;
+    readstat_metadata_ok_callback       metadata_ok;
 } readstat_writer_callbacks_t;
 
 /* You'll need to define one of these to get going. Should return # bytes written,
@@ -405,6 +480,7 @@ typedef struct readstat_writer_s {
     int                         row_count;
     int                         current_row;
     char                        file_label[100];
+    char                        table_name[33];
     const readstat_variable_t  *fweight_variable;
 
     readstat_writer_callbacks_t callbacks;
@@ -443,8 +519,10 @@ void readstat_variable_set_label_set(readstat_variable_t *variable, readstat_lab
 void readstat_variable_set_measure(readstat_variable_t *variable, readstat_measure_t measure);
 void readstat_variable_set_alignment(readstat_variable_t *variable, readstat_alignment_t alignment);
 void readstat_variable_set_display_width(readstat_variable_t *variable, int display_width);
-void readstat_variable_add_missing_double_value(readstat_variable_t *variable, double value);
-void readstat_variable_add_missing_double_range(readstat_variable_t *variable, double lo, double hi);
+readstat_error_t readstat_variable_add_missing_double_value(readstat_variable_t *variable, double value);
+readstat_error_t readstat_variable_add_missing_double_range(readstat_variable_t *variable, double lo, double hi);
+readstat_error_t readstat_variable_add_missing_string_value(readstat_variable_t *variable, const char *value);
+readstat_error_t readstat_variable_add_missing_string_range(readstat_variable_t *variable, const char *lo, const char *hi);
 readstat_variable_t *readstat_get_variable(readstat_writer_t *writer, int index);
 
 // "Notes" appear in the file metadata. In SPSS these are stored as
@@ -465,12 +543,22 @@ readstat_string_ref_t *readstat_get_string_ref(readstat_writer_t *writer, int in
 readstat_error_t readstat_writer_set_file_label(readstat_writer_t *writer, const char *file_label);
 readstat_error_t readstat_writer_set_file_timestamp(readstat_writer_t *writer, time_t timestamp);
 readstat_error_t readstat_writer_set_fweight_variable(readstat_writer_t *writer, const readstat_variable_t *variable);
+
 readstat_error_t readstat_writer_set_file_format_version(readstat_writer_t *writer, 
-        long file_format_version); // e.g. 104-118 for DTA; 5 or 8 for SAS Transport
+        uint8_t file_format_version);
+// e.g. 104-119 for DTA; 5 or 8 for SAS Transport.
+// SAV files support 2 or 3, where 3 is equivalent to setting 
+// readstat_writer_set_compression(READSTAT_COMPRESS_BINARY)
+
+readstat_error_t readstat_writer_set_table_name(readstat_writer_t *writer, const char *table_name);
+// Only used in XPORT files at the moment (defaults to DATASET)
+
 readstat_error_t readstat_writer_set_file_format_is_64bit(readstat_writer_t *writer,
         int is_64bit); // applies only to SAS files; defaults to 1=true
 readstat_error_t readstat_writer_set_compression(readstat_writer_t *writer,
-        readstat_compress_t compression); // applies only to SAS and SAV files
+        readstat_compress_t compression); 
+        // READSTAT_COMPRESS_BINARY is supported only with SAV files (i.e. ZSAV files)
+        // READSTAT_COMPRESS_ROWS is supported only with sas7bdat and SAV files
 
 // Optional error handler
 readstat_error_t readstat_writer_set_error_handler(readstat_writer_t *writer, 
@@ -484,6 +572,10 @@ readstat_error_t readstat_begin_writing_sas7bdat(readstat_writer_t *writer, void
 readstat_error_t readstat_begin_writing_sav(readstat_writer_t *writer, void *user_ctx, long row_count);
 readstat_error_t readstat_begin_writing_xport(readstat_writer_t *writer, void *user_ctx, long row_count);
 
+// Optional, file-specific validation routines, to be called AFTER readstat_begin_writing_XXX
+readstat_error_t readstat_validate_metadata(readstat_writer_t *writer);
+readstat_error_t readstat_validate_variable(readstat_writer_t *writer, const readstat_variable_t *variable);
+
 // Start a row of data (that is, a case or observation)
 readstat_error_t readstat_begin_row(readstat_writer_t *writer);
 
