@@ -297,16 +297,12 @@ static readstat_error_t xport_write_member_record_v8(readstat_writer_t *writer,
     if (writer->table_name[0])
         ds_name = writer->table_name;
 
-    if ((retval = sas_validate_name(ds_name, 32)) != READSTAT_OK)
-        goto cleanup;
-
     snprintf(member_header, sizeof(member_header),
             "%-8.8s" "%-32.32s"   "%-8.8s"   "%-8.8s" "%-8.8s" "%16.16s",
             "SAS",   ds_name, "SASDATA", "6.06",  "bsd4.2", timestamp);
 
     retval = xport_write_record(writer, member_header);
 
-cleanup:
     return retval;
 }
 
@@ -321,16 +317,12 @@ static readstat_error_t xport_write_member_record(readstat_writer_t *writer,
     if (writer->table_name[0])
         ds_name = writer->table_name;
 
-    if ((retval = sas_validate_name(ds_name, 8)) != READSTAT_OK)
-        goto cleanup;
-
     snprintf(member_header, sizeof(member_header),
             "%-8.8s" "%-8.8s" "%-8.8s"   "%-8.8s" "%-8.8s"  "%-24.24s" "%16.16s",
             "SAS",   ds_name, "SASDATA", "6.06",  "bsd4.2", "",        timestamp);
 
     retval = xport_write_record(writer, member_header);
 
-cleanup:
     return retval;
 }
 
@@ -365,9 +357,12 @@ static readstat_error_t xport_write_obs_header_record(readstat_writer_t *writer)
     return xport_write_header_record(writer, &xrecord);
 }
 
-static void xport_format_timestamp(char *output, size_t output_len, time_t timestamp) {
+static readstat_error_t xport_format_timestamp(char *output, size_t output_len, time_t timestamp) {
     struct tm *ts = localtime(&timestamp);
 
+    if (!ts)
+        return READSTAT_ERROR_BAD_TIMESTAMP_VALUE;
+
     snprintf(output, output_len, 
             "%02d%3.3s%02d:%02d:%02d:%02d",
             (unsigned int)ts->tm_mday % 100, 
@@ -377,13 +372,18 @@ static void xport_format_timestamp(char *output, size_t output_len, time_t times
             (unsigned int)ts->tm_min % 100,
             (unsigned int)ts->tm_sec % 100
             );
+
+    return READSTAT_OK;
 }
 
 static readstat_error_t xport_begin_data(void *writer_ctx) {
     readstat_writer_t *writer = (readstat_writer_t *)writer_ctx;
     readstat_error_t retval = READSTAT_OK;
     char timestamp[17];
-    xport_format_timestamp(timestamp, sizeof(timestamp), writer->timestamp);
+
+    retval = xport_format_timestamp(timestamp, sizeof(timestamp), writer->timestamp);
+    if (retval != READSTAT_OK)
+        goto cleanup;
 
     retval = xport_write_first_header_record(writer);
     if (retval != READSTAT_OK)
@@ -502,14 +502,30 @@ static readstat_error_t xport_write_missing_tagged(void *row, const readstat_var
     return error;
 }
 
+static readstat_error_t xport_metadata_ok(void *writer_ctx) {
+    readstat_writer_t *writer = (readstat_writer_t *)writer_ctx;
+
+    if (writer->version != 5 && writer->version != 8)
+        return READSTAT_ERROR_UNSUPPORTED_FILE_FORMAT_VERSION;
+
+    if (writer->table_name[0]) {
+        if (writer->version == 8) {
+            return sas_validate_name(writer->table_name, 32);
+        }
+        if (writer->version == 5) {
+            return sas_validate_name(writer->table_name, 8);
+        }
+    }
+
+    return READSTAT_OK;
+}
+
 readstat_error_t readstat_begin_writing_xport(readstat_writer_t *writer, void *user_ctx, long row_count) {
 
     if (writer->version == 0)
         writer->version = XPORT_DEFAULT_VERISON;
 
-    if (writer->version != 5 && writer->version != 8)
-        return READSTAT_ERROR_UNSUPPORTED_FILE_FORMAT_VERSION;
-
+    writer->callbacks.metadata_ok = &xport_metadata_ok;
     writer->callbacks.write_int8 = &xport_write_int8;
     writer->callbacks.write_int16 = &xport_write_int16;
     writer->callbacks.write_int32 = &xport_write_int32;
