@@ -20,7 +20,9 @@
 #define DTA_DEFAULT_FORMAT_FLOAT   "9.0g"
 #define DTA_DEFAULT_FORMAT_DOUBLE "10.0g"
 
-#define DTA_DEFAULT_FILE_VERSION   118
+#define DTA_FILE_VERSION_MIN     104
+#define DTA_FILE_VERSION_MAX     119
+#define DTA_FILE_VERSION_DEFAULT 118
 
 #define DTA_OLD_MAX_WIDTH    128
 #define DTA_111_MAX_WIDTH    244
@@ -107,6 +109,17 @@ static readstat_error_t dta_emit_header_time_stamp(readstat_writer_t *writer, dt
     char months[][4] = { 
         "Jan", "Feb", "Mar", "Apr", "May", "Jun",
         "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
+
+    if (!time_s) {
+        error = READSTAT_ERROR_BAD_TIMESTAMP_VALUE;
+        goto cleanup;
+    }
+
+    if (!timestamp) {
+        error = READSTAT_ERROR_MALLOC;
+        goto cleanup;
+    }
+
     uint8_t actual_timestamp_len = snprintf(timestamp, ctx->timestamp_len, "%02d %3s %04d %02d:%02d",
             time_s->tm_mday, months[time_s->tm_mon], time_s->tm_year + 1900,
             time_s->tm_hour, time_s->tm_min);
@@ -262,10 +275,11 @@ cleanup:
     return error;
 }
 
-static readstat_error_t dta_validate_name(const char *name, size_t max_len) {
+static readstat_error_t dta_validate_name_chars(const char *name, int unicode) {
+    /* TODO check Unicode class */
     int j;
     for (j=0; name[j]; j++) {
-        if (name[j] != '_' &&
+        if ((name[j] > 0 || !unicode) && name[j] != '_' &&
                 !(name[j] >= 'a' && name[j] <= 'z') &&
                 !(name[j] >= 'A' && name[j] <= 'Z') &&
                 !(name[j] >= '0' && name[j] <= '9')) {
@@ -273,11 +287,15 @@ static readstat_error_t dta_validate_name(const char *name, size_t max_len) {
         }
     }
     char first_char = name[0];
-    if (first_char != '_' &&
+    if ((first_char > 0 || !unicode) && first_char != '_' &&
             !(first_char >= 'a' && first_char <= 'z') &&
             !(first_char >= 'A' && first_char <= 'Z')) {
         return READSTAT_ERROR_NAME_BEGINS_WITH_ILLEGAL_CHARACTER;
     }
+    return READSTAT_OK;
+}
+
+static readstat_error_t dta_validate_name_unreserved(const char *name) {
     if (strcmp(name, "_all") == 0 || strcmp(name, "_b") == 0 ||
             strcmp(name, "byte") == 0 || strcmp(name, "_coef") == 0 ||
             strcmp(name, "_cons") == 0 || strcmp(name, "double") == 0 ||
@@ -294,22 +312,34 @@ static readstat_error_t dta_validate_name(const char *name, size_t max_len) {
     if (sscanf(name, "str%d", &len) == 1)
         return READSTAT_ERROR_NAME_IS_RESERVED_WORD;
 
-    if (strlen(name) > max_len)
-        return READSTAT_ERROR_NAME_IS_TOO_LONG;
-
     return READSTAT_OK;
 }
 
-static readstat_error_t dta_old_variable_ok(readstat_variable_t *variable) {
-    return dta_validate_name(readstat_variable_get_name(variable), DTA_OLD_MAX_NAME_LEN);
+static readstat_error_t dta_validate_name(const char *name, int unicode, size_t max_len) {
+    readstat_error_t error = READSTAT_OK;
+
+    if (strlen(name) > max_len)
+        return READSTAT_ERROR_NAME_IS_TOO_LONG;
+
+    if (strlen(name) == 0)
+        return READSTAT_ERROR_NAME_IS_ZERO_LENGTH;
+
+    if ((error = dta_validate_name_chars(name, unicode)) != READSTAT_OK)
+        return error;
+
+    return dta_validate_name_unreserved(name);
 }
 
-static readstat_error_t dta_110_variable_ok(readstat_variable_t *variable) {
-    return dta_validate_name(readstat_variable_get_name(variable), DTA_110_MAX_NAME_LEN);
+static readstat_error_t dta_old_variable_ok(const readstat_variable_t *variable) {
+    return dta_validate_name(readstat_variable_get_name(variable), 0, DTA_OLD_MAX_NAME_LEN);
 }
 
-static readstat_error_t dta_118_variable_ok(readstat_variable_t *variable) {
-    return dta_validate_name(readstat_variable_get_name(variable), DTA_118_MAX_NAME_LEN);
+static readstat_error_t dta_110_variable_ok(const readstat_variable_t *variable) {
+    return dta_validate_name(readstat_variable_get_name(variable), 0, DTA_110_MAX_NAME_LEN);
+}
+
+static readstat_error_t dta_118_variable_ok(const readstat_variable_t *variable) {
+    return dta_validate_name(readstat_variable_get_name(variable), 1, DTA_118_MAX_NAME_LEN);
 }
 
 static readstat_error_t dta_emit_varlist(readstat_writer_t *writer, dta_ctx_t *ctx) {
@@ -844,26 +874,9 @@ static size_t dta_old_variable_width(readstat_type_t type, size_t user_width) {
     return dta_numeric_variable_width(type, user_width);
 }
 
-static readstat_error_t dta_emit_header(readstat_writer_t *writer, dta_ctx_t *ctx,
-        dta_header_t *header) {
+static readstat_error_t dta_emit_xmlish_header(readstat_writer_t *writer, dta_ctx_t *ctx) {
     readstat_error_t error = READSTAT_OK;
 
-    if (!ctx->file_is_xmlish) {
-        error = readstat_write_bytes(writer, header, sizeof(dta_header_t));
-        if (error != READSTAT_OK)
-            goto cleanup;
-
-        error = dta_emit_header_data_label(writer, ctx);
-        if (error != READSTAT_OK)
-            goto cleanup;
-
-        error = dta_emit_header_time_stamp(writer, ctx);
-        if (error != READSTAT_OK)
-            goto cleanup;
-
-        return READSTAT_OK;
-    }
-
     if ((error = dta_write_tag(writer, ctx, "<stata_dta>")) != READSTAT_OK)
         goto cleanup;
 
@@ -871,27 +884,36 @@ static readstat_error_t dta_emit_header(readstat_writer_t *writer, dta_ctx_t *ct
         goto cleanup;
 
     char release[128];
-    snprintf(release, sizeof(release), "<release>%d</release>", header->ds_format);
+    snprintf(release, sizeof(release), "<release>%ld</release>", writer->version);
     if ((error = readstat_write_string(writer, release)) != READSTAT_OK)
         goto cleanup;
 
     error = dta_write_chunk(writer, ctx, "<byteorder>",
-            (header->byteorder == DTA_HILO) ? "MSF" : "LSF", sizeof("MSF")-1,
+            machine_is_little_endian() ? "LSF" : "MSF", sizeof("MSF")-1,
             "</byteorder>");
     if (error != READSTAT_OK)
         goto cleanup;
 
-    error = dta_write_chunk(writer, ctx, "<K>", &header->nvar, sizeof(int16_t), "</K>");
-    if (error != READSTAT_OK)
-        goto cleanup;
-
-    if (header->ds_format >= 118) {
-        int64_t nobs = header->nobs;
-        error = dta_write_chunk(writer, ctx, "<N>", &nobs, sizeof(int64_t), "</N>");
+    if (writer->version >= 119) {
+        uint32_t nvar = writer->variables_count;
+        error = dta_write_chunk(writer, ctx, "<K>", &nvar, sizeof(uint32_t), "</K>");
         if (error != READSTAT_OK)
             goto cleanup;
     } else {
-        error = dta_write_chunk(writer, ctx, "<N>", &header->nobs, sizeof(int32_t), "</N>");
+        uint16_t nvar = writer->variables_count;
+        error = dta_write_chunk(writer, ctx, "<K>", &nvar, sizeof(uint16_t), "</K>");
+        if (error != READSTAT_OK)
+            goto cleanup;
+    }
+
+    if (writer->version >= 118) {
+        uint64_t nobs = writer->row_count;
+        error = dta_write_chunk(writer, ctx, "<N>", &nobs, sizeof(uint64_t), "</N>");
+        if (error != READSTAT_OK)
+            goto cleanup;
+    } else {
+        uint32_t nobs = writer->row_count;
+        error = dta_write_chunk(writer, ctx, "<N>", &nobs, sizeof(uint32_t), "</N>");
         if (error != READSTAT_OK)
             goto cleanup;
     }
@@ -911,6 +933,38 @@ cleanup:
     return error;
 }
 
+static readstat_error_t dta_emit_header(readstat_writer_t *writer, dta_ctx_t *ctx) {
+    if (ctx->file_is_xmlish) {
+        return dta_emit_xmlish_header(writer, ctx);
+    }
+    readstat_error_t error = READSTAT_OK;
+    dta_header_t header = {0};
+
+    header.ds_format = writer->version;
+    header.byteorder = machine_is_little_endian() ? DTA_LOHI : DTA_HILO;
+    header.filetype  = 0x01;
+    header.unused    = 0x00;
+    header.nvar      = writer->variables_count;
+    header.nobs      = writer->row_count;
+
+    if (writer->variables_count > 32767) {
+        error = READSTAT_ERROR_TOO_MANY_COLUMNS;
+        goto cleanup;
+    }
+
+    if ((error = readstat_write_bytes(writer, &header, sizeof(dta_header_t))) != READSTAT_OK)
+        goto cleanup;
+
+    if ((error = dta_emit_header_data_label(writer, ctx)) != READSTAT_OK)
+        goto cleanup;
+
+    if ((error = dta_emit_header_time_stamp(writer, ctx)) != READSTAT_OK)
+        goto cleanup;
+
+cleanup:
+    return READSTAT_OK;
+}
+
 static size_t dta_measure_tag(dta_ctx_t *ctx, const char *tag) {
     if (!ctx->file_is_xmlish)
         return 0;
@@ -1062,20 +1116,13 @@ static readstat_error_t dta_begin_data(void *writer_ctx) {
         return READSTAT_ERROR_WRITER_NOT_INITIALIZED;
     
     dta_ctx_t *ctx = dta_ctx_alloc(NULL);
-    dta_header_t header = {0};
 
-    header.ds_format = writer->version;
-    header.byteorder = machine_is_little_endian() ? DTA_LOHI : DTA_HILO;
-    header.filetype  = 0x01;
-    header.unused    = 0x00;
-    header.nvar      = writer->variables_count;
-    header.nobs      = writer->row_count;
-
-    error = dta_ctx_init(ctx, header.nvar, header.nobs, header.byteorder, header.ds_format, NULL, NULL);
+    error = dta_ctx_init(ctx, writer->variables_count, writer->row_count,
+            machine_is_little_endian() ? DTA_LOHI : DTA_HILO, writer->version, NULL, NULL);
     if (error != READSTAT_OK)
         goto cleanup;
     
-    error = dta_emit_header(writer, ctx, &header);
+    error = dta_emit_header(writer, ctx);
     if (error != READSTAT_OK)
         goto cleanup;
 
@@ -1335,18 +1382,25 @@ static void dta_module_ctx_free(void *module_ctx) {
     dta_ctx_free(module_ctx);
 }
 
-readstat_error_t readstat_begin_writing_dta(readstat_writer_t *writer, void *user_ctx, long row_count) {
+readstat_error_t dta_metadata_ok(void *writer_ctx) {
+    readstat_writer_t *writer = (readstat_writer_t *)writer_ctx;
 
     if (writer->compression != READSTAT_COMPRESS_NONE)
         return READSTAT_ERROR_UNSUPPORTED_COMPRESSION;
 
-    if (writer->version == 0)
-        writer->version = DTA_DEFAULT_FILE_VERSION;
-
-    if (writer->version >= 119 || writer->version < 104) {
+    if (writer->version > DTA_FILE_VERSION_MAX || writer->version < DTA_FILE_VERSION_MIN)
         return READSTAT_ERROR_UNSUPPORTED_FILE_FORMAT_VERSION;
-    }
-    
+
+    return READSTAT_OK;
+}
+
+readstat_error_t readstat_begin_writing_dta(readstat_writer_t *writer, void *user_ctx, long row_count) {
+
+    if (writer->version == 0)
+        writer->version = DTA_FILE_VERSION_DEFAULT;
+
+    writer->callbacks.metadata_ok = &dta_metadata_ok;
+
     if (writer->version >= 117) {
         writer->callbacks.variable_width = &dta_117_variable_width;
     } else if (writer->version >= 111) {
@@ -1363,7 +1417,7 @@ readstat_error_t readstat_begin_writing_dta(readstat_writer_t *writer, void *use
         writer->callbacks.variable_ok = &dta_old_variable_ok;
     }
 
-    if (writer->version == 118) {
+    if (writer->version >= 118) {
         writer->callbacks.write_string_ref = &dta_118_write_string_ref;
     } else if (writer->version == 117) {
         writer->callbacks.write_string_ref = &dta_117_write_string_ref;
