@@ -12,7 +12,7 @@ test_that("zap_labels returns variables not of class('labelled') unmodified", {
 })
 
 test_that("zap_labels is applied to every column in data frame", {
-  df <- tibble::data_frame(x = 1:10, y = labelled(10:1, c("good" = 1)))
+  df <- tibble::tibble(x = 1:10, y = labelled(10:1, c("good" = 1)))
   expect_equal(zap_labels(df)$y, 10:1)
 })
 
